/*
 * Copyright (c) 2005-2007 Extreme! Lab, Indiana University. All rights reserved.
 *
 * This software is open source. See the bottom of this file for the license.
 *
 * $Id: 
 */
package edu.indiana.extreme.xbaya.component.registry;

import java.io.File;
import java.io.IOException;
import java.util.List;

import edu.indiana.extreme.xbaya.XBayaConstants;
import edu.indiana.extreme.xbaya.XBayaRuntimeException;
import edu.indiana.extreme.xbaya.component.ComponentException;
import edu.indiana.extreme.xbaya.component.gui.ComponentTreeNode;
import edu.indiana.extreme.xbaya.component.ws.WSComponent;
import edu.indiana.extreme.xbaya.component.ws.WSComponentFactory;
import edu.indiana.extreme.xbaya.util.IOUtil;

/**
 * @author Satoshi Shirasuna
 */
public class LocalComponentRegistry extends ComponentRegistry {

    // private static final MLogger logger = MLogger.getLogger();

    private File directory;

    /**
     * Creates a LocalComponentRegistryClient
     * 
     * @param directory
     */
    public LocalComponentRegistry(String directory) {
        this(new File(directory));
    }

    /**
     * Creates a WebComponentRegistryClient
     * 
     * @param directory
     *            The path of the directory that contains the component files.
     */
    public LocalComponentRegistry(File directory) {
        this.directory = directory;
    }

    /**
     * @see edu.indiana.extreme.xbaya.component.registry.ComponentRegistry#getName()
     */
    @Override
    public String getName() {
        return this.directory.toString();
    }

    /**
     * Returns a ComponentTree.
     * 
     * @return The ComponentTree
     * @throws ComponentRegistryException
     */
    @Override
    public ComponentTreeNode getComponentTree()
            throws ComponentRegistryException {
        try {
            ComponentTreeNode componentTree = getComponentTree(this.directory);
            componentTree.setComponentRegistry(this);
            return componentTree;
        } catch (RuntimeException e) {
            throw new ComponentRegistryException(e);
        }
    }

    /**
     * Returns a component of a specified name.
     * 
     * @param name
     *            The name of the component. The name here is a file path
     *            relative to the directory.
     * @return The component of a specified name
     * @throws ComponentException
     * @throws ComponentRegistryException
     */
    public WSComponent getComponent(String name) throws ComponentException,
            ComponentRegistryException {
        // This method is used only by unit tests.
        File file = new File(this.directory, name);
        return getComponents(file).get(0);
    }

    /**
     * @param file
     * @return The list of components defined in the specified file.
     * @throws ComponentException
     * @throws ComponentRegistryException
     */
    public List<WSComponent> getComponents(File file)
            throws ComponentException, ComponentRegistryException {
        try {
            String compString = IOUtil.readFileToString(file);
            List<WSComponent> components = WSComponentFactory
                    .createComponents(compString);
            return components;
        } catch (IOException e) {
            throw new ComponentRegistryException(e);
        }
    }

    private ComponentTreeNode getComponentTree(File dir) {
        if (!dir.isDirectory()) {
            throw new XBayaRuntimeException(dir + "is not a directory.");
        }

        boolean found = false;
        ComponentTreeNode tree = new ComponentTreeNode(dir.getName());
        for (File file : dir.listFiles()) {
            String fileName = file.getName();
            if (file.isDirectory()) {
                ComponentTreeNode subTree = getComponentTree(file);
                if (subTree != null) {
                    found = true;
                    tree.add(subTree);
                }
            } else if (fileName.endsWith(XBayaConstants.XML_SUFFIX)
                    || fileName.endsWith(XBayaConstants.WSDL_SUFFIX)
                    || fileName.endsWith(XBayaConstants.WSDL_SUFFIX2)) {
                found = true;
                LocalComponentReference componentReference = new LocalComponentReference(
                        file.getName(), file, this);
                ComponentTreeNode treeLeaf = new ComponentTreeNode(
                        componentReference);
                treeLeaf.setAllowsChildren(false);
                tree.add(treeLeaf);
            }
        }
        if (!found) {
            // Doesn't show a directory that doesn't have any components.
            tree = null;
        }
        return tree;
    }
}

/*
 * Indiana University Extreme! Lab Software License, Version 1.2
 * 
 * Copyright (c) 2005-2007 The Trustees of Indiana University. All rights
 * reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 * 
 * 2) All redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 * 
 * 3) Any documentation included with all redistributions must include the
 * following acknowledgement:
 * 
 * "This product includes software developed by the Indiana University Extreme!
 * Lab. For further information please visit http://www.extreme.indiana.edu/"
 * 
 * Alternatively, this acknowledgment may appear in the software itself, and
 * wherever such third-party acknowledgments normally appear.
 * 
 * 4) The name "Indiana University" or "Indiana University Extreme! Lab" shall
 * not be used to endorse or promote products derived from this software without
 * prior written permission from Indiana University. For written permission,
 * please contact http://www.extreme.indiana.edu/.
 * 
 * 5) Products derived from this software may not use "Indiana University" name
 * nor may "Indiana University" appear in their name, without prior written
 * permission of the Indiana University.
 * 
 * Indiana University provides no reassurances that the source code provided
 * does not infringe the patent or any other intellectual property rights of any
 * other entity. Indiana University disclaims any liability to any recipient for
 * claims brought by any other entity based on infringement of intellectual
 * property rights or otherwise.
 * 
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 * NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS.
 * INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS",
 * "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.
 * LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION
 * GENERATED USING SOFTWARE.
 */
